@@ -45,6 +45,8 @@
 
 #define BLOCKSIZE 10240
 #define MAXGZ 1024*512*sizeof(gint)
+#define CONTROLTAR "control.tar"
+#define DATATAR "data.tar"
 
 typedef enum {
 	DV_AR_FAIL = -1,
@@ -55,6 +57,7 @@ typedef enum {
 	DV_ERR_CORRUPT_FILE,
 	DV_ERR_NOT_DPKG,
 	DV_ERR_COMPRESS,
+	DV_ERR_BAD_MEMBER_NAME,
 } DVError;
 
 /* details of the files within the .deb */
@@ -297,6 +300,10 @@ static void dv_show_error (DVError dve, GtkWidget * widget) {
 			fmt = _("Unable to decompress file within package.");
 			break;
 		}
+		case DV_ERR_BAD_MEMBER_NAME : {
+			fmt = _("Not a valid member name.");
+			break;
+		}
 	}
 	if (deb->ar_message) {
 		fmt = g_strconcat (fmt, "\n", deb->ar_message, NULL);
@@ -425,7 +432,7 @@ static gulong parseheaderlength (const gchar *inh, size_t len) {
 
 static gboolean prepare_contents (GFile * gfile, DVContents * deb) {
 	gsize G_GNUC_UNUSED bytesread, header, seek;
-	GError * result;
+	GError *result;
 	GFileInputStream * filestream;
 	struct ar_hdr debian_binary;
 	struct ar_hdr controlgz;
@@ -499,8 +506,44 @@ static gboolean prepare_contents (GFile * gfile, DVContents * deb) {
 		g_clear_error (&result);
 	}
 	g_free (infobuf);
-	header = sizeof(struct ar_hdr);
-	bytesread = g_input_stream_read (deb->stream, &controlgz, sizeof (struct ar_hdr), NULL, &result);
+
+	// control.tar
+
+	// Member after debian-binary usually will be control.tar* but
+	// could be files beginning with an underscore _
+
+	// If the next member read begins with an underscore,
+	// get its file length and and skip that many bytes 
+	// Continue looping until control.tar* is reached
+
+	// Any other member name is incorrect .deb format and will cause an error
+
+	while ((bytesread = g_input_stream_read(deb->stream, &controlgz, sizeof (struct ar_hdr), NULL, &result)) > 0) {
+		// If control.tar* are the next bytes, break
+		if (memcmp(controlgz.ar_name, CONTROLTAR, strlen(CONTROLTAR)) == 0) {
+			break;
+		}
+		// If it's an underscore member, get its file size,
+		// skip that many bytes (+1 if odd number file size), and continue looping
+		else if (controlgz.ar_name[0] == '_') {
+			memberlen = parseheaderlength (controlgz.ar_size, sizeof (controlgz.ar_size));
+			if (!memberlen) {
+				dv_show_error (DV_ERR_CORRUPT_FILE, deb->parent);
+			}
+			g_input_stream_skip(deb->stream, memberlen + (memberlen & 1), NULL, &result);
+			if (result) {
+				g_warning ("%s", result->message);
+				g_clear_error (&result);
+			}
+			continue;
+		}
+		// If not an underscore member or control.tar*, invalid member name
+		else {
+			dv_show_error(DV_ERR_BAD_MEMBER_NAME, deb->parent);
+			break;
+		}
+	}
+
 	if (result) {
 		g_warning ("%s", result->message);
 		g_clear_error (&result);
@@ -577,8 +620,46 @@ static gboolean prepare_contents (GFile * gfile, DVContents * deb) {
 			return FALSE;
 		}
 	}
-	header = sizeof(struct ar_hdr);
-	bytesread = g_input_stream_read (deb->stream, &datagz, sizeof(struct ar_hdr), NULL, &result);
+	
+	// data.tar
+
+	// Member after control.tar* usually will be data.tar* but
+	// could be files beginning with an underscore _
+
+	// If the next member read begins with an underscore,
+	// get its file length and and skip that many bytes 
+	// Continue looping until data.tar* is reached
+
+	// If the next member read is data.tar*, continue processing it
+
+	// Any other member name is incorrect .deb format and will cause an error
+
+	while ((bytesread = g_input_stream_read(deb->stream, &datagz, sizeof (struct ar_hdr), NULL, &result)) > 0) {
+		// If data.tar* are the next bytes, break
+		if (memcmp(datagz.ar_name, DATATAR, strlen(DATATAR)) == 0) {
+			break;
+		}
+		// If it's an underscore member, get its file size,
+		// skip that many bytes (+1 if odd number file size), and continue looping
+		else if (datagz.ar_name[0] == '_') {
+			memberlen = parseheaderlength (datagz.ar_size, sizeof (datagz.ar_size));
+			if (!memberlen) {
+				dv_show_error (DV_ERR_CORRUPT_FILE, deb->parent);
+			}
+			g_input_stream_skip(deb->stream, memberlen + (memberlen & 1), NULL, &result);
+			if (result) {
+				g_warning ("%s", result->message);
+				g_clear_error (&result);
+			}
+			continue;
+		}
+		// If not an underscore member or data.tar*, invalid member name
+		else {
+			dv_show_error(DV_ERR_BAD_MEMBER_NAME, deb->parent);
+			break;
+		}
+	}
+
 	if (result) {
 		g_warning ("Data tarball read error: %s", result->message);
 		g_clear_error (&result);
